------------------------------------------------------------------------
--- Manager for CGI servers.
--- 
--- @author Michael Hanus
--- @version March 2007
------------------------------------------------------------------------

import ReadShowTerm
import System
import IOExts

mainShow   = cmdForAllServers "Status of " "show"

mainSketch = cmdForAllServers "Sketch status of " "sketch"

mainClean  = cmdForAllServers "Clean status of " "clean" >>
             getAndCleanRegistry >> done

mainStop   = cmdForAllServers "Stopping cgi server " "stop" >>
             getAndCleanRegistry >> done

mainKill   = doForAllServers "Killing process of cgi server "
                             (\s->system ("kill -9 "++show (fst s))) >>
             getAndCleanRegistry >> done

showAllActiveServers :: IO ()
showAllActiveServers = do
  regs <- getAndCleanRegistry
  --print regs
  putStrLn "Currently active cgi script servers:"
  mapIO_ printServer regs
 where
  printServer (pid,progname) = putStrLn $ progname ++ " (PID: "++show pid++")"

doForAllServers :: String -> ((Int,String) -> IO _) -> IO ()
doForAllServers cmt action = do
  regs <- getAndCleanRegistry
  mapIO_ doForServer regs
 where
  doForServer (pid,progname) = do
    putStrLn $ cmt ++ progname++":"
    action (pid,progname)

cmdForAllServers :: String -> String -> IO ()
cmdForAllServers cmt actiontag =
 doForAllServers cmt (\s -> system (snd s++" "++actiontag) >> done)

-- Get the registry with active processes and clean up the registry file.
getAndCleanRegistry :: IO [(Int,String)]
getAndCleanRegistry = exclusiveIO (cgiServerRegistry++".lock") $ do
  regs <- readQTermListFile cgiServerRegistry
  aregs <- mapIO (\ (pid,pname) -> isActivePID pid >>= \pidactive ->
                   return (if pidactive then [(pid,pname)] else [])) regs
  let cregs = concat aregs
  if cregs==regs
   then done
   else writeFile cgiServerRegistry (concatMap (\reg->show reg++"\n") cregs)
  return cregs

-- Is an integer the pid of an existing process?
isActivePID :: Int -> IO Bool
isActivePID pid = do
  mypid <- getPID
  let tmp = "/tmp/tmp_pakcs_registry_"++show mypid
  system ("ps -p "++show pid++" | fgrep "++show pid++" > "++tmp)
  pr <- readCompleteFile tmp
  system ("rm "++tmp)
  return (not (null pr))

-- The name of the registry file.
cgiServerRegistry = "/tmp/PAKCS_CGISERVERS"

